;Curlicue Fractals
;BlitzPlus version
;by Andres Amaya Jr.
;2016.05.11

;===========================================================
;	Press [C] to change colors used to plot curlicue
;
;	Press [S] to Save BMP of curlicue to disk
;
;	Press [ESC] to Exit
;
;	Press any other key for another curlicue
;===========================================================

; *** Links for info on Curlicue Fractals ***
;http://oolong.co.uk/curlicue.htm
;https://scratch.mit.edu/projects/1027160/
;https://scratch.mit.edu/projects/1132887/
;https://scratch.mit.edu/projects/1259540/
;https://en.wikipedia.org/wiki/Spiral     (Cornu Spiral)
;http://www.mathematische-basteleien.de/spiral.htm   (Clothoide)
;http://www.codeproject.com/Questions/580130/Drawingplusaplusclothoidplusspline


AppTitle "Curlicue Fractals"
Global sw, sh
sw = 960 : sh = 720

Graphics sw, sh, 32, 2
SetBuffer BackBuffer()
SeedRnd MilliSecs()

Dim hx8$(17),rr%(24),gg%(24),bb%(24),hxc%(5,24)
palette()


Local x#, y#, f#, i#, r#, angle#, scale#, iters#
Local count%, ci%, cx%, cy%, hexColor%, x1%, y1%
Local defaultName$, savePic$
Local imgOut%, result%, paws%, colorFlag%

paws = CreateTimer(1)

iters = 260000
cIdx% = 0
While KeyHit(1)=0
	Cls

	If colorFlag = 0 Then
		r = Rnd(0.000001, 9.999999)
		scale = scalePlot(iters, r)
	End If

	f=0.0 : x=0.0 : y=0.0 : ci=0
	cx = sw/2 : cy = sh/2 : count = 0

	LockBuffer()
	For i = 0 To iters Step 0.5
		If count =0 Then
			ci=(ci+1) Mod 24
			hexColor = hxc(cIdx, ci)
		End If
		f=f+r
		angle = (f*i) Mod 360.0
		x = Cos(angle) - x
		y = Sin(angle) - y
		x1 = Int(x * scale) + cx
		y1 = Int(y * scale) + cy
		count = (count + 1) Mod 5200
		If x1>=0 And x1<sw And y1>=0 And y1<sh Then WritePixelFast(x1,y1,hexColor)
		If KeyHit(1) Then End
	Next
	UnlockBuffer()
	Color 255,255,255
	Text 2,sh-15,"Seed: "+r
	colorFlag = 0
	Flip

	WaitKey()

	If KeyHit(31) Then ; Save bmp to disk when [S] key pressed
		curlicueName$ = "Curlicue_"+r+"_"+cIdx+".bmp"
		imgOut = CreateImage(sw,sh)
		GrabImage imgOut%,0,0
		result = SaveImage(imgOut, curlicueName$)
		If result = 1 Then
			Color 255,255,255
			Rect cx-153,cy-18, 306, 36, True
			Color 255,0,0
			Rect cx-150,cy-15, 300, 30, True
			Color 255,255,255
			Text cx ,cy- 8, curlicueName$+" saved",True
			Flip
			;st=MilliSecs()+1500: While st>MilliSecs() : Wend
			WaitTimer(paws)
			WaitTimer(paws)
		Else
			Notify "Save Image Failed."
		End If
		FreeImage imgOut
	End If
	
	If KeyHit(46) Then ;Change color when [C] key pressed
		cIdx = (cIdx + 1) Mod 6
		colorFlag% = 1
	End If
	
Wend

End

Function scalePlot#(iMax%, seed#)
	Local fInc#, ang#, scaleX#, scaleY#, mult#
	Local j#, a#, b#, px#, py#
	Local wide#, high#
	Local centx% = sw Shr 1
	Local centy% = sh Shr 1
	Local minx# =  9999.
	Local maxx# = -9999.
	Local miny# =  9999.
	Local maxy# = -9999
	
	For j = 0 To iMax Step 0.5
		fInc = fInc + seed ;seed determines the appearance of curlicue fractal
		ang = (fInc * j) Mod 360.0
		a = Cos(ang) - a
		b = Sin(ang) - b
		px = a + centx
		py = b + centy
		
		If px < minx Then minx = px	;find upper left X coord of plot
		If py < miny Then miny = py	;find upper left Y coord of plot
		If px > maxx Then maxx = px	;find lower right X coord of plot
		If py > maxy Then maxy = py	;find lower right Y coord of plot
		If KeyHit(1) Then End
	Next	
	wide = maxx - minx	;width of plotted points
	high = maxy - miny	;height of plotted points
	
	scaleX = sw/wide	;multiply width of plot by scaleX to plot at screen width
	scaleY = sh/high	;multiply height of plot by scaleY to plot at screen height

	;find smaller of two scale factors to stay within window dimensions
	If scaleX > scaleY Then mult = scaleY Else mult = scaleX
	
	Return mult*0.992	;make sure all pixels are on screen
End Function
	
Function h2d$(hx$)
	Local hi%, lo%
	hx$ = Upper(hx$)
	hi = Asc( Left(hx$,1))-48
	lo = Asc(Right(hx$,1))-48
	If hi>10 Then hi=hi-7
	If lo>10 Then lo=lo-7
	Return hi Shl 4 Or lo
End Function


Function palette%()
	Local scheme%, i%, j%, count%
	Local triplet$
	;=============== Color 0 (rainbow) ========================
	hx8$( 0)="0000FF4000FF7D00FFBE00FFFF00FFFF00BEFF007DFF0040"
	hx8$( 1)="FF0000FF4000FF7D00FFBE00FFFF00BEFF007DFF0040FF00"
	hx8$( 2)="00FF0000FF4000FF7D00FFBE00FFFF00BEFF007DFF0040FF"

	;================ Color 1 (red) ===========================
	hx8$( 3)="FF0000FF2200C31F0D9A1F0DAF3B15A4411AA4411AA8481F"
	hx8$( 4)="B0572ACD681FB76933E7561FF37D29C7863EC78844C36013"
	hx8$( 5)="A4411AA4411AAF3B159A1F0DC31F0DFF2200FF00009A1F0D"

	;================ Color 2 (yellow) ========================
	hx8$( 6)="EFEF78F1F167F4F456F6F645F8F833FAFA22FDFD11FFFF00"
	hx8$( 7)="FDFD11FDE70FFED00CFEBA0AFEA307FE8D05FF7602FF6000"
	hx8$( 8)="FF7602FD8810FB991FF9AB2DF7BC3CF5CE4AF3DF59F1F167"

	;================ Color 3 (green) =========================
	hx8$( 9)="005000006900008200009B0000B40000CD0000E60000FF00"
	hx8$(10)="10FF400EEB3A0BD63509C22F07AD2905992302841E007018"
	hx8$(11)="10C02011B71F12AE1E13A51D159B1B16921A178919188018"

	;================ Color 4 (blue) ==========================
	hx8$(12)="0D1EFF0D1ED10821AD1F376D29427E29427E3E598F4A6398"
	hx8$(13)="4A6398576EA26A81B36A81B37994C18BB4D96F9AEB8FBADC"
	hx8$(14)="6F9AE794BEE094BEE06F9ADE90B1DD90B1DD3E66D5142C6D"

	;================ Color 5 (purple) ========================
	hx8$(15)="80004092055BA40977B60E92C912ADDB17C8ED1BE4FF20FF"
	hx8$(16)="FF60FFF660E1ED60C4E460A6DB6089D2606BC9604EC06030"
	hx8$(17)="C9604EBF524CB4454AAA37489F2946951B448A0E42800040"


	For scheme% = 0 To 5
		count = 0
		For i = 0 To 2
			For j = 1 To 43 Step 6
				triplet$ = Mid(hx8$(scheme*3+i),j,6)
				rr(count) = h2d$(Mid(triplet$,1,2))
				gg(count) = h2d$(Mid(triplet$,3,2))
				bb(count) = h2d$(Mid(triplet$,5,2))
				hxc(scheme, count) = rr(count) Shl 16 Or gg(count) Shl 8 Or bb(count)
				count = count + 1
			Next
		Next
	Next
	Return 1
End Function